unit MainWbMl;

interface

uses Windows, SysUtils, Classes, HTTPApp, SimpleMail;

const sTitLog ='Login zum WebMail-Interface';   // diese Seitentitel knnten
      sTitBox ='WebMail-Mailbox';               // auch in der INI-Datei stehen
      sTitShow='Webmail-Nachricht';
      sTitSnd ='WebMail-Sendeergebnis';
      sTitAdm ='WebMail-Administration';
      sTitErr ='WebMail-Fehler';

type TUserData = Record                    // Datensatzabbild der Userdatenbank
  User: String; PassU: String; Name:  String; eMail: String;
  Pop3: String; Smtp: String; Konto: String; Pass: String;
end;

type
  TWebMod = class(TWebModule)
    Content: TPageProducer;
    procedure WebModCreate(Sender: TObject);
    procedure WebModBeforeDispatch(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure WebModAfterDispatch(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure WebModActionLogin(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModActionShowMail(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModActionDelete(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModActionSend(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModActionAdmin(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModActionDefault(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  private
    Smtp: TSmtpObject;
    Pop3: TPop3Object;
    User: TUserData;
    ScriptName,               // logischer Pfad und Name der DLL
    ScriptPathP,              // physischer Pfad zut DLL
    ScriptPathL,              // logischer Pfad auf dem Server
    ScriptIni,                // INI-Datei im neben der DLL Webmail-Verzeichnis
    DataBase: String;         // Pfad und Name der Userdatenbank
    PageTitle,                // Titel der zu generierenden Seite
    HTMLDetail,               // bergabestring an Seitenproducer
    TextDetail,               // bergabestring an Seitenproducer
    AddDetail: String;        // bergabestring an Seitenproducer
    nDetail: Integer;         // bergabewert   an Seitenproducer
    procedure ProducePage(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
    procedure ProduceLogin(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
    procedure ProduceShowMail(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
    procedure ProduceMailbox(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
    procedure ProduceSendOK(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
    procedure ProduceAdmin(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
    function GetUserData: Boolean;          // holt Daten aus der Userdatenbank
    function UpdateDataBase(UserDat: TUserData): Boolean;  // Aktualisierung DB
    function EnCrypt(const sIn: String): String;  // verschlsselt einen String
    function DeCrypt(const sIn: String): String;  // entschlsselt einen String
    function HtmlLink(const sRef, sText: String): String; //generiert Hyperlink
  end;

var
  WebMod: TWebMod;

implementation

{$R *.DFM}

procedure TWebMod.WebModCreate(Sender: TObject);
var F: array[0..MAX_PATH-1] of Char;
begin
 SetString(ScriptName,F,GetModuleFilename(hInstance,F,SizeOf(F)));
 ScriptPathP:=ExtractFilePath(ScriptName);           // physischer Pfad der DLL
 DataBase  :=ScriptPathP+'DataBase.txt';     // Zugangsdatenbank (hier als TXT)
 ScriptIni :=ChangeFileExt(ScriptName,'.ini');  // INI-Datei fr evt. Parameter
end;

procedure TWebMod.WebModBeforeDispatch(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var i: Integer;
begin
 ScriptName:=Request.ScriptName;           // logischer Serverpfad und DLL-Name
 i:=Length(ScriptName)-1; while (i>0) and (ScriptName[i]<>'/') do Dec(i);
 ScriptPathL:=Copy(ScriptName,1,i-1);     // logischen Serverpfad heraustrennen
 PageTitle:=sTitErr; HTMLDetail:='illegaler Aufruf'; nDetail:=0;  // vorbeugend
end;

procedure TWebMod.WebModAfterDispatch(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var Templ: TMemoryStream; Res: TResourceStream; PageTempl: String;
begin                     // in jedem Fall Master-Template webmail.htm ausgeben
 Templ:=TMemoryStream.Create();
 try
  Templ.LoadFromFile(ScriptPathP+'webmail.htm'); // Master-Design der Antworten
 except               // falls nicht zu finden: Notreserven aus Ressource holen
  Res:=TResourceStream.Create(HInstance,'MainTemplate',RT_RCDATA);
  Templ.LoadFromStream(Res);
  Res.Free;
  Res:=TResourceStream.Create(HInstance,'LetterGif',RT_RCDATA);
  try Res.SaveToFile(ScriptPathP+'MAILICON.GIF') except end;
  Res.Free;
 end;
 SetString(PageTempl,PChar(Templ.Memory),Templ.Size);
 Templ.Free;
 Content.OnHTMLTag:=ProducePage;
 Response.Content:=Content.ContentFromString(PageTempl);
end;

procedure TWebMod.ProducePage(Sender: TObject; Tag: TTag;  // bei AfterDispatch
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String;
begin
 sTag:=UpperCase(TagString);
 if sTag='SCRIPTDIR' then  ReplaceText:=ScriptPathL
 else if sTag='TITLE' then ReplaceText:=PageTitle
 else if sTag='CONTENT' then ReplaceText:=HTMLDetail
 else if sTag='DATE' then ReplaceText:=FormatDateTime('dddd, yy-mm-dd',Now())
 else if sTag='TIME' then ReplaceText:=FormatDateTime('hh:nn:ss',Now());
end;

procedure TWebMod.WebModActionDefault(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
//Reaktion auf unklare Aufrufe: NICHTS tun (Ausnahme: GET ohne Pfad wird Login)
begin
 if (Request.PathInfo='') and (Request.MethodType=mtGet) then
  WebModActionLogin(Sender, Request, Response, Handled);
end;

procedure TWebMod.WebModActionLogin(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
// Login (kalt) von auen oder (warm, mit MailBox-Zugang) vom Loginformular
var sVar, sLnk: String; nErg, nI: Integer;
begin
 if Request.MethodType=mtGet then begin        // Pfad: /login oder leerer Pfad
  Content.HtmlFile:=ScriptPathP+'login.htm';    // Login-Anmelde-Formular laden
  Content.OnHTMLTag:=ProduceLogin;
  try
   HTMLDetail:=Content.Content
  except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
  PageTitle:=sTitLog; 
 end
 else if Request.MethodType=mtPost then begin   // Anmeldung vom Login-Formular
  User.User:=Request.ContentFields.Values['User'];
  User.PassU:=Request.ContentFields.Values['Pass'];
  if Request.PathInfo='/mailbox' then                 // nach Lschen von Mails
   User.PassU:=DeCrypt(Request.ContentFields.Values['PassU']);
  if GetUserData then begin    // Userdatenbank abgecheckt und Userdaten geholt
   Pop3:=TPOP3Object.Create;
   Pop3.Server:=User.Pop3; Pop3.Konto:=User.Konto; Pop3.Pass:=User.Pass;
   try
    nErg:=Pop3.CheckAccount;                     //Account am POPServer checken
    if nErg=POP3_MAILOK then begin        // Mailserver hat positiv geantwortet
     sVar:='';
     for nI:=0 to Pop3.Count-1 do begin        // ber alle vorhandenen e-Mails
      try nErg:=StrToInt(Pop3.GetSize(nI)) div 1024 +1 except nErg:=0 end;
      // Subject der e-Mail in Form eines Hyperlinks mit dem Pfad /showmail mit
      // den Parametern Username, Passwort, Nummer, POP-Server fr die sptere
      // Anzeige der e-Mail generieren
      sLnk:=HtmlLink(Format('%s/showmail?s=%s&u=%s&p=%s&n=%d',[ScriptName,
       URLCode(Pop3.Server), URLCode(Pop3.Konto), EnCrypt(Pop3.Pass), nI+1]),
       Pop3.GetSubject(nI));
      // Zeile der Inbox-Tabelle mit Absender, Subjekt, Datum u. Gre erzeugen
      sVar:=sVar+Format(
       ' <TR><TD>%s</TD><TD>%s</TD><TD>%s</TD><TD>%s</TD><TD>%s</TD></TR>'#13#10,
       [Format('<INPUT TYPE="checkbox" NAME="Del" VALUE="%d">',[nI+1]),
        HtmlMaskLtGt(Pop3.GetFrom(nI)), sLnk, Pop3.GetDate(nI),
        Format('%d KB',[nErg])]);
     end;
     nDetail:=Pop3.Count;                                 // Anzahl der e-Mails
     if nDetail>0 then  sVar:=sVar+                // Button "Lschen" erzeugen
      ' <TR><TD COLSPAN="5"><INPUT TYPE="submit" VALUE="L&ouml;schen"></TD></TR>';
     HTMLDetail:=sVar;                              // Zeilen der Inbox-Tabelle
    end else begin                         // Mailserver hat Antwort verweigert
     case nErg of
      POP3_NOPASS  : sVar:='Passwort nicht korrekt';
      POP3_NOKONTO : sVar:='Konto nicht vorhanden';
      SOCK_NOSERVER: sVar:='Mailserver nicht bereit';
      SOCK_NONET   : sVar:='keine Netzwerkverbindung';
      POP3_NOPARAMS: sVar:='fehlende Userdaten';
     else sVar:='undefinierter Fehler' end;
     HTMLDetail:=Format('<TR><TD COLSPAN=5>'+
      '<H4>W&auml;hrend der &Uuml;bertragung ist ein Fehler aufgetreten!</H4>'+
      'Der Fehler lautet: <FONT COLOR="#CC0033">%s</FONT></TD></TR>',[sVar]);
    end;
   except {CheckAccount undefiniert} end;
   Pop3.Free;
   Content.HtmlFile:=ScriptPathP+'mailbox.htm';       // Mailbox-Formular laden
   Content.OnHTMLTag:=ProduceMailbox;
   try
    HTMLDetail:=Content.Content; // direkten Mailbox-Inhalt nach Muster erzeugen
   except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
   PageTitle:=sTitBox+' f&uuml;r '+User.eMail;
  end else begin      // Login falsch, zurck zum Login-Formular, neuer Versuch
   HTMLDetail:='<H3><FONT COLOR="#CC0033">Anmeldefehler!</FONT></H3>';
   try
    nDetail:=StrToInt(Request.ContentFields.Values['Attempt'])+1 // Hidden-Feld
   except nDetail:=0 end;
   if nDetail<3 then begin                         // erste/zweite Wiederholung
    {HTMLDetail:=HTMLDetail+Format('Sie haben noch %d Versuche.',[3-nDetail]);}
    Content.HtmlFile:=ScriptPathP+'login.htm';
    Content.OnHTMLTag:=ProduceLogin;
    try
     HTMLDetail:=HTMLDetail+Content.Content; //Fehlermeldung plus Login-Formular
    except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
    PageTitle:=sTitLog;
   end else begin                                // mehr als drei Loginversuche
    HTMLDetail:=HTMLDetail+'Login gesperrt.';
    PageTitle:=sTitErr;
   end;
  end; // if GetUserData
 end; // if GET/POST
end;

procedure TWebMod.ProduceLogin(Sender: TObject; Tag: TTag;    // Login-Formular
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String;
begin
 sTag:=UpperCase(TagString);
 if sTag='SCRIPT' then begin
  if Request.PathInfo='/admin' then ReplaceText:=ScriptName+'/admin'
  else ReplaceText:=ScriptName+'/login';
 end
 else if sTag='ATTEMPT' then ReplaceText:=IntToStr(nDetail);
end;

procedure TWebMod.ProduceMailbox(Sender: TObject; Tag: TTag;    // Mailbox-Page
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String;
begin
 sTag:=UpperCase(TagString);
 if sTag='SCRIPT' then ReplaceText:=ScriptName
 else if sTag='INMAILS' then ReplaceText:=IntToStr(nDetail)   // Anzahl e-Mails
 else if sTag='LINES' then ReplaceText:=HTMLDetail      // Tabellenzeilen InBox
 else if sTag='SENDER' then ReplaceText:=User.Name   // Absender im Postausgang
 else if sTag='USERDATA' then Replacetext:=Format(     // geheime Absenderdaten
  ' <INPUT TYPE="hidden" NAME="User"  VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="PassU" VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="Pop3"  VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="Smtp"  VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="Name"  VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="eMail" VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="Konto" VALUE="%s">'#13#10+
  ' <INPUT TYPE="hidden" NAME="Pass"  VALUE="%s">',
  [User.User,Encrypt(User.PassU),User.Pop3,User.Smtp,
                          User.Name,User.eMail,User.Konto,Encrypt(User.Pass)]);
end;

procedure TWebMod.WebModActionShowMail(Sender: TObject; // eine e-Mail anzeigen
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var nErg, i: Integer; AttachNames: TStringList; SRec: TSearchRec;
begin
 Pop3:=TPOP3Object.Create;
 Pop3.Server:=Request.QueryFields.Values['s'];    // Servername aus QueryString
 Pop3.Konto:=Request.QueryFields.Values['u'];       // Username aus QueryString
 Pop3.Pass:=DeCrypt(Request.QueryFields.Values['p']);  // PassW aus QueryString
 try i:=StrToInt(Request.QueryFields.Values['n']) except i:=1 end;   // Mail-Nr
 try
  nErg:=Pop3.RetrMail(i);                 // holt die Mail Nr. i vom Mailserver
  if nErg=POP3_MAILOK then begin          // Mail i erfolgreich heruntergeladen
   nDetail:=i;
   // hier eventuell die gesamte Mail in einem Userverzeichnis abspeichern
   HtmlDetail:=HtmlMaskLtGt(Pop3.GetHeader);           // "<" und ">" maskieren
   TextDetail:=Pop3.GetMailText;                       // eigentlicher MailText
   AddDetail:='';
   AttachNames:=TStringList.Create;
   AttachNames.Text:=Pop3.GetAttachNames;              // Namen der Attachments
   for i:=0 to AttachNames.Count-1 do begin            // ber alle Attachments
    if FindFirst(ScriptPathP+'inbox\*.*', faAnyFile, SRec)=0 then begin
     while FindNext(SRec)=0 do                 // alle alten Attachments prfen
      if (SRec.Attr and faDirectory)<>faDirectory then
       if FileDateToDateTime(SRec.Time)+0.1 < Now() then
        DeleteFile(ScriptPathP+'inbox\'+sRec.Name);  // lter als 2 Stunden weg
     FindClose(SRec);
    end else try MkDir(ScriptPathP+'inbox') except end;
    Pop3.StoreAttachment(i, ScriptPathP+'inbox\'+AttachNames[i]);
    AddDetail:=AddDetail+' '+HtmlLink(
                ScriptPathL+'/inbox/'+AttachNames[i],AttachNames[i])+#13#10;
   end;
   AttachNames.Free;
  end else begin                            // Mail i nicht erfolgreich geladen
   case nErg of
    POP3_NOPASS  : HTMLDetail:='Passwort nicht korrekt';
    POP3_NOKONTO : HTMLDetail:='Konto nicht vorhanden';
    SOCK_NOSERVER: HTMLDetail:='Mailserver nicht bereit';
    SOCK_NONET   : HTMLDetail:='keine Netzwerkverbindung';
    POP3_NOPARAMS: HTMLDetail:='fehlende Userdaten';
   else HTMLDetail:='undefinierter Fehler' end;
   HTMLDetail:=Format('<TR><TD COLSPAN=5>'+
    '<H4>W&auml;hrend der &Uuml;bertragung ist ein Fehler aufgetreten!</H4>'+
    'Der Fehler lautet: <FONT COLOR="#CC0033">%s</FONT></TD></TR>',[HTMLDetail]);
  end;
 except end;
 Pop3.Free;
 Content.HtmlFile:=ScriptPathP+'showmail.htm';       // Mailausgabe-Seite laden
 Content.OnHTMLTag:=ProduceShowMail;
 try
  HTMLDetail:=Content.Content
 except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
 PageTitle:=sTitShow+' '+IntToStr(nDetail);
end;

procedure TWebMod.ProduceShowMail(Sender: TObject; Tag: TTag;
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String;
begin
 sTag:=UpperCase(TagString);
 if sTag='HEADER' then ReplaceText:=HtmlDetail          // Mailheader einsetzen
 else if sTag='BODY' then ReplaceText:=TextDetail         // Mailtext einsetzen
 else if sTag='ATTACHMENT' then ReplaceText:=AddDetail;     // Attachment-Namen
end;

procedure TWebMod.WebModActionDelete(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var i: Integer; sMails: String;
begin
 sMails:='';
 for i:=0 to Request.ContentFields.Count-1 do       // Checkboxen "Del=" suchen
  if UpperCase(Copy(Request.ContentFields[i],1,4))='DEL=' then
   sMails:=sMails+Copy(Request.ContentFields[i],5,9)+#13#10;
 if Length(sMails)>0 then begin
  Pop3:=TPOP3Object.Create;
  Pop3.Server:=Request.ContentFields.Values['Pop3'];              // Hidden-Feld
  Pop3.Konto:=Request.ContentFields.Values['Konto'];              // Hidden-Feld
  Pop3.Pass:=DeCrypt(Request.ContentFields.Values['Pass']);       // Hidden-Feld
  try Pop3.DeleteMails(sMails) except end;
  Pop3.Free;
 end;
 WebModActionLogin(Sender, Request, Response, Handled);
end;

procedure TWebMod.WebModActionSend(Sender: TObject;         // e-Mail versenden
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var nErg: Integer;
begin
 Smtp:=TSmtpObject.Create;
 Smtp.Server:=Request.ContentFields.Values['Smtp'];              // Hidden-Feld
 Smtp.MailTo:=Request.ContentFields.Values['MailTo'];           // Formularfeld
 Smtp.From:=Request.ContentFields.Values['eMail'];               // Hidden-Feld
 Smtp.Sender:=Request.ContentFields.Values['Sender'];           // Formularfeld
 Smtp.Subject:=Request.ContentFields.Values['Subject'];         // Formularfeld
 Smtp.SetContent(Request.ContentFields.Values['Text']);         // Formularfeld
 nErg:=Smtp.SendMail;          //e-Mail senden mit Funktion aus Unit SimpleMail
 if nErg=SMTP_MAILOK then begin                        // Mail korrekt gesendet
  // eventuell e-Mail in einem User-Ordner "gesendet" auf dem Server speichern
  Content.HtmlFile:=ScriptPathP+'sendok.htm';
  Content.OnHTMLTag:=ProduceSendOK;
  try
   HTMLDetail:=Content.Content
  except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
 end else begin            // Senden fehlerhaft, Fehlermeldungsseite generieren
  case nErg of
   SMTP_TRANSERR: HTMLDetail:='&Uuml;bertragungsfehler';
   SOCK_NOSERVER: HTMLDetail:='Mailserver nicht bereit';
   SOCK_NONET   : HTMLDetail:='keine Netzwerkverbindung';
   SMTP_NOPARAMS: HTMLDetail:='fehlende e-Mailangaben';
  else HTMLDetail:='undefinierter Fehler' end;
  HTMLDetail:=Format(
   '<H4>Bei der &Uuml;bermittlung der Daten ist ein Fehler aufgetreten!</H4>'+
   'Die Daten wurden <B>nicht</B> weitergeleitet.<BR>'+
   'Der Fehler lautet: <FONT COLOR="#CC0033">%s</FONT>'+
   '<P>[ <SMALL>%s</SMALL> ]</P>',
   [HtmlDetail,HtmlLink('javascript:history.back()','zur&uuml;ck zur Mailbox')]);
 end;
 Smtp.Free;                                               // nicht vergessen!!!
 PageTitle:=sTitSnd;
end;

procedure TWebMod.ProduceSendOK(Sender: TObject; Tag: TTag; // Sendebesttigung
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String; nLn: Integer;
begin
 sTag:=UpperCase(TagString); nLn:=0; ReplaceText:='';
 if sTag='SENDER' then ReplaceText:=Smtp.Sender
 else if sTag='MAILTO' then ReplaceText:=Smtp.MailTo
 else if sTag='SUBJECT' then ReplaceText:=Smtp.Subject
 else if sTag='TEXT' then while nLn<Smtp.ContentLines do begin
  ReplaceText:=ReplaceText+Smtp.GetContentLine(nLn)+'<BR>'; Inc(nLn);
 end;
end;

// ----------------------------------------------------------------------------

procedure TWebMod.WebModActionAdmin(Sender: TObject;     // Userdaten verwalten
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
begin
 if Request.MethodType=mtGet then begin
  Content.HtmlFile:=ScriptPathP+'login.htm';            // Login-Formular laden
  Content.OnHTMLTag:=ProduceLogin;
  try
   HTMLDetail:=Content.Content
  except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
  PageTitle:='Zugang zu '+sTitAdm;
 end else if Request.MethodType=mtPost then begin // Reaktion auf ein Formuular
  User.User:=Request.ContentFields.Values['User'];
  User.PassU:=Request.ContentFields.Values['Pass'];
  if Request.Query<>'data' then begin           // Admin-Formular zum Ausfllen
   if GetUserData then begin                // User vorhanden und Login korrekt
    HTMLDetail:=
     '<P>Sie k&ouml;nnen nun Ihre bisherigen Zugangsdaten &auml;ndern:</P>';
   end else begin                               // unbekannter User, neuanlegen
    HTMLDetail:='<P>Tragen Sie hier Ihre Zugangsdaten f&uuml;r e-Mail ein:</P>';
    User.User:=''; User.PassU:=''; User.Name:=''; User.eMail:='';
    User.Pop3:=''; User.Smtp:=''; User.Konto:=''; User.Pass:='';
   end;
   Content.HtmlFile:=ScriptPathP+'admin.htm';      // Admin-Datenformular laden
   PageTitle:=sTitAdm;
  end else begin     // Querystring war "data", Formular-Anmeldedaten speichern
   User.PassU:=Request.ContentFields.Values['PassU'];
   User.Name:=Request.ContentFields.Values['Sender'];
   User.eMail:=Request.ContentFields.Values['eMail'];
   User.Pop3:=Request.ContentFields.Values['Pop3'];
   User.Smtp:=Request.ContentFields.Values['Smtp'];
   User.Konto:=Request.ContentFields.Values['Konto'];
   User.Pass:=Request.ContentFields.Values['Pass'];
   if UpdateDataBase(User) then begin                // erfolgreich gespeichert
    HTMLDetail:='';
    Content.HtmlFile:=ScriptPathP+'adminok.htm'; //Admin-Datenbesttigung laden
    PageTitle:=sTitAdm+' (Best&auml;tigung)';
   end else begin                                        // speichern miglckt
    HTMLDetail:='<P>Ihre Daten konnten <B>nicht</B> gespeichert werden.</P>';
    Content.HtmlFile:=''; Content.HTMLDoc.Clear;
    PageTitle:='Fehler '+sTitAdm;
   end;
  end;
  Content.OnHTMLTag:=ProduceAdmin;
  try
   HTMLDetail:=HTMLDetail+#13#10+Content.Content;
  except HtmlDetail:=ExtractFileName(Content.HtmlFile)+' fehlt!' end;
 end; // if GET/POST
end;

procedure TWebMod.ProduceAdmin(Sender: TObject; Tag: TTag;  // Datenbesttigung
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String;
begin
 sTag:=UpperCase(TagString);
 if sTag='SCRIPT' then begin
  if Request.Query='' then ReplaceText:=ScriptName+'/admin?data'
  else ReplaceText:=ScriptName;
 end
 else if sTag='USER' then ReplaceText:=User.User
 else if sTag='PASSU' then ReplaceText:=User.PassU
 else if sTag='SENDER' then ReplaceText:=User.Name
 else if sTag='EMAIL' then ReplaceText:=User.eMail
 else if sTag='POP3' then ReplaceText:=User.Pop3
 else if sTag='SMTP' then ReplaceText:=User.Smtp
 else if sTag='KONTO' then ReplaceText:=User.Konto
 else if sTag='PASS' then ReplaceText:=User.Pass;
end;

// ----------------------------------------------------------------------------

function TWebMod.GetUserData: Boolean;
// prft Usernamen und Passwort und fllt UserDat-Record mit allen Userdaten
// aus der Datenbank.
// Diese Routine bitte der eigenen Datenbank anpassen!!
var Data: TStringList; bOK: Boolean; nLn,nP: Integer; sLn, sUser: String;
begin
 bOK:=false; sUser:=LowerCase(User.User);
 if (Length(sUser)>0) and (Length(User.PassU)>0) then begin
  Data:=TStringList.Create;
  try Data.LoadFromFile(DataBase) except end;
  nLn:=0;
  while (not bOK) and (nLn<Data.Count) do begin
   sLn:=Data[nLn];             // eine Zeile als Semikolon-getrennter Datensatz
   nP:=Pos(';',sLn);
   if nP>0 then if sUser=Copy(sLn,1,nP-1) then begin           // User gefunden
    Delete(sLn,1,nP);                                         // um User krzen
    nP:=Pos(';',sLn);
    if nP>0 then if User.PassU=DeCrypt(Copy(sLn,1,nP-1)) then begin   // Pass OK
     Delete(sLn,1,nP);                                        // um Pass krzen
     nP:=Pos(';',sLn);
     if nP>0 then User.Name:=Copy(sLn,1,nP-1);                 // Name einlesen
     Delete(sLn,1,nP);                                       // um Namen krzen
     nP:=Pos(';',sLn);
     if nP>0 then User.eMail:=Copy(sLn,1,nP-1);       // eMail-Adresse einlesen
     Delete(sLn,1,nP);                                       // um eMail krzen
     nP:=Pos(';',sLn);
     if nP>0 then User.Pop3:=Copy(sLn,1,nP-1);           // Pop-Server einlesen
     Delete(sLn,1,nP);                                         // um Pop krzen
     nP:=Pos(';',sLn);
     if nP>0 then User.Smtp:=Copy(sLn,1,nP-1);          // Smtp-Server einlesen
     Delete(sLn,1,nP);                                         // um Pop krzen
     nP:=Pos(';',sLn);
     if nP>0 then User.Konto:=Copy(sLn,1,nP-1);           // Kontoname einlesen
     Delete(sLn,1,nP);                                       // um Konto krzen
     nP:=Pos(';',sLn);
     if nP>0 then Delete(sLn,nP,999);                       // Rest abschneiden
     if Length(sLn)>0 then begin User.Pass:=DeCrypt(sLn); bOK:=true; end;
    end; // if Pass
   end; // if sUser
   Inc(nLn);
  end; //while
  Data.Free;
 end; // if Length>0
 GetUserData:=bOK;
end;

function TWebMod.UpdateDataBase(UserDat: TUserData): Boolean; // DatenbankUpdate
var Data: TStringList; bOK, bFound: Boolean; nLn,nP: Integer; sLn, sUser: String;
begin
 bOK:=false; sUser:=LowerCase(UserDat.User);
 if (Length(sUser)>0) and (Length(UserDat.PassU)>0) then begin
  Data:=TStringList.Create;
  try Data.LoadFromFile(DataBase) except end;
  nLn:=0; bFound:=false;
  while (not bFound) and (nLn<Data.Count) do begin
   sLn:=Data.Strings[nLn];
   nP:=Pos(';',sLn);
   if nP>0 then if sUser=Copy(sLn,1,nP-1) then begin           // User gefunden
    Delete(sLn,1,nP);                                         // um User krzen
    nP:=Pos(';',sLn);
    if nP>0 then if UserDat.PassU=DeCrypt(Copy(sLn,1,nP-1)) then begin //Pass OK
     Data.Delete(nLn); bFound:=true;       // alte Datensatz-Datenzeile lschen
    end;
   end;
   Inc(nLn)
  end;
  with UserDat do
  if (Length(Name)>0) and (Length(eMail)>0) and (Length(Pop3)>0)
                and (Length(Smtp)>0) and (Length(Konto)>0) and (Length(Pass)>0)
  then begin                 // neue Datensatz-Datenzeile erzeugen und anhngen
   Data.Add(Format('%s;%s;%s;%s;%s;%s;%s;%s;',
            [sUser,EnCrypt(PassU),Name,eMail,Pop3,Smtp,Konto,EnCrypt(Pass)]));
   try Data.SaveToFile(DataBase); bOK:=true; except end;
  end;
  Data.Free;
 end;
 UpdateDataBase:=bOK;
end;

function TWebMod.EnCrypt(const sIn: String): String;     //String verschlsseln
var s: String; i: Integer;
begin
 s:='';
 for i:=1 to Length(sIn) do s:=Format('%x',[Ord(sIn[i])-13])+s;
 EnCrypt:=s;
end;

function TWebMod.DeCrypt(const sIn: String): String;     //String entschlsseln
var s: String; i: Integer;
begin
 s:='';
 for i:=1 to Length(sIn) div 2 do
  s:=Chr((HxCharToInt(sIn[i+i-1])shl 4)+HxCharToInt(sIn[i+i])+13)+s;
 DeCrypt:=s;
end;

function TWebMod.HtmlLink(const sRef, sText: String): String; //Link generieren
var sTx: String;
begin
 if Length(sText)>0 then sTx:=sText else sTx:=sRef;
 HtmlLink:='<A HREF="'+sRef+'">'+sTx+'</A>';
end;

end.
